# -*- coding: UTF-8 -*-

import os
import sys

if sys.version_info.major == 3:
    from dbfpy3 import dbf
else:
    from dbfpy import dbf


class DataHeader(object):
    def __init__(self, workDir, charset, currentSales):
        """
        @param currentSales - выгрузка текущих продаж
        """
        self.workDirectory = workDir
        self.hasCrashedTable = False
        self.currentSales = currentSales
        self.cashSailDB = self.getCashSail()
        self.cashDcrdDB = self.getCashDcrd()
        self.saleFiles = set((self.cashSailDB.name, self.cashDcrdDB.name))
        if not self.currentSales:
            self.cashDiscDB = self.getCashDisc()
            self.currestsDB = self.getCurRests()
            self.curMoneyDB = self.getCurMoney()
            self.cashPayDB = self.getCashPay()
            self.saleFiles.add(self.cashDiscDB.name)
            self.saleFiles.add(self.currestsDB.name)
            self.saleFiles.add(self.curMoneyDB.name)
            self.saleFiles.add(self.cashPayDB.name)

    def close(self):
        self.cashSailDB.close()
        self.cashDcrdDB.close()
        if not self.currentSales:
            self.cashDiscDB.close()
            self.currestsDB.close()
            self.curMoneyDB.close()
            self.cashPayDB.close()

    def getCashSail(self):
        db = None
        fileName = self.workDirectory + '/cashsail.dbf'
        if os.path.exists(fileName) and not self.hasCrashedTable:
            try:
                obj = file(fileName, "r+b")
                db = dbf.Dbf(obj)
                return db
            except Exception as e:
                self.hasCrashedTable = True
        db = dbf.Dbf(fileName, new=True)

        fields_list = [
            ("SHOPINDEX", "C", 6),
            ("CASHNUMBER", "C", 6),
            ("ZNUMBER", "N", 6),
            ("CHECKNUMBE", "N", 6),
            ("ID", "N", 6),
            ("DATE", "D", 8),
            ("TIME", "N", 6),
            ("CARDARTICU", "C", 30),
            ("CARDSIZE", "C", 10),
            ("QUANTITY", "N", 16, 6),
            ("PRICERUB", "N", 16, 2),
            ("PRICECUR", "N", 16, 2),
            ("TOTALRUB", "N", 16, 2),
            ("TOTALCUR", "N", 16, 2),
            ("DEPARTMENT", "N", 6),
            ("CASHER", "N", 6),
            ("USINGINDEX", "N", 6),
            ("REPLACE", "N", 6),
            ("OPERATION", "N", 6),
            ("CREDCARDIN", "N", 6),
            ("DISCCLIIND", "N", 6),
            ("LINKED", "N", 6),
        ]
        dbf_fields = [self.get_dbf_field(*f) for f in fields_list]
        self.add_fields_to_dbf_reader(db, dbf_fields)
        return db

    def getCashDisc(self):
        db = None
        fileName = self.workDirectory + '/cashdisc.dbf'
        if os.path.isfile(fileName) and not self.hasCrashedTable:
            try:
                obj = file(fileName, "r+b")
                db = dbf.Dbf(obj)
                return db
            except:
                self.hasCrashedTable = True
        db = dbf.Dbf(fileName, new=True)
        fields_list = [
            ("SHOPINDEX", "C", 6),
            ("CASHNUMBER", "C", 6),
            ("ZNUMBER", "N", 6),
            ("CHECKNUMBE", "N", 6),
            ("ID", "N", 6),
            ("DISCOUNTIN", "N", 6),
            ("DISCOUNTPR", "N", 16, 2),
            ("DISCOUNTRU", "N", 16, 2),
            ("DISCOUNTCU", "N", 16, 2)
        ]
        dbf_fields = [self.get_dbf_field(*f) for f in fields_list]
        self.add_fields_to_dbf_reader(db, dbf_fields)
        return db

    def getCurRests(self):
        db = None
        fileName = self.workDirectory + '/currests.dbf'
        if os.path.isfile(fileName) and not self.hasCrashedTable:
            try:
                obj = file(fileName, "r+b")
                db = dbf.Dbf(obj)
                return db
            except Exception as e:
                self.hasCrashedTable = True
        db = dbf.Dbf(fileName, new=True)
        fields_list = [
            ("SHOPINDEX", "C", 6),
            ("CASHNUMBER", "C", 6),
            ("ZNUMBER", "N", 6),
            ("DATE", "D"),
            ("SALE", "N", 16, 2),
            ("RETURN", "N", 16, 2),
            ("STORNO", "N", 16, 2),
            ("STORNRET", "N", 16, 2),
            ("RESULT", "N", 16, 2),
            ("WASOUTPUT", "N", 6)
        ]
        dbf_fields = [self.get_dbf_field(*f) for f in fields_list]
        self.add_fields_to_dbf_reader(db, dbf_fields)
        return db

    def getCurMoney(self):
        db = None
        fileName = self.workDirectory + '/curmoney.dbf'
        if os.path.isfile(fileName) and not self.hasCrashedTable:
            try:
                obj = file(fileName, "r+b")
                db = dbf.Dbf(obj)
                return db
            except:
                self.hasCrashedTable = True
        db = dbf.Dbf(fileName, new=True)
        fields_list = (
            ("SHOPINDEX", "C", 6),
            ("CASHNUMBER", "C", 6),
            ("ZNUMBER", "N", 6),
            ("CURIDX", "N", 6),
            ("BEFORE", "N", 16, 2),
            ("BEFOREIN", "N", 16, 2),
            ("BEFOREOUT", "N", 16, 2),
            ("PERIODIN", "N", 16, 2),
            ("PERIODOUT", "N", 16, 2),
            ("AFTER", "N", 16, 2),
        )
        dbf_fields = [self.get_dbf_field(*f) for f in fields_list]
        self.add_fields_to_dbf_reader(db, dbf_fields)
        return db

    def getCashDcrd(self):
        db = None
        fileName = self.workDirectory + '/cashdcrd.dbf'
        if os.path.isfile(fileName) and not self.hasCrashedTable:
            try:
                obj = file(fileName, "r+b")
                db = dbf.Dbf(obj)
                return db
            except:
                self.hasCrashedTable = True
        db = dbf.Dbf(fileName, new=True)
        fields_list = (
            ("SHOPINDEX", "C", 6),
            ("CASHNUMBER", "C", 6),
            ("ZNUMBER", "N", 6),
            ("CHECKNUMBE", "N", 6),
            ("CARDTYPE", "N", 6),
            ("CARDNUMBER", "C", 22),
            ("DISCOUNTRU", "N", 16, 2),
            ("DISCOUNTCU", "N", 16, 2)
        )
        dbf_fields = [self.get_dbf_field(*f) for f in fields_list]
        self.add_fields_to_dbf_reader(db, dbf_fields)
        return db

    def getCashGoodDB(self):
        db = None
        fileName = self.workDirectory + '/cashgood.dbf'
        if os.path.isfile(fileName) and not self.hasCrashedTable:
            try:
                obj = file(fileName, "r+b")
                db = dbf.Dbf(obj)
                return db
            except:
                self.hasCrashedTable = True
        db = dbf.Dbf(fileName, new=True)
        fields_list = (
            ("SHOPINDEX", "C", 6),
            ("CASHNUMBER", "C", 6),
            ("ZNUMBER", "N", 6),
            ("REPLACE", "N", 6),
            ("USINGINDEX", "N", 6),
            ("CARDARTICU", "C", 30),
            ("CARDSIZE", "C", 10),
            ("QUANTITY", "N", 16, 6),
            ("SUMMARUB", "N", 16, 2),
            ("SUMMACUR", "N", 16, 2)
        )
        dbf_fields = [self.get_dbf_field(*f) for f in fields_list]
        self.add_fields_to_dbf_reader(db, dbf_fields)
        return db

    def getCashPay(self):
        db = None
        fileName = self.workDirectory + '/cashpay.dbf'
        if os.path.isfile(fileName) and not self.hasCrashedTable:
            try:
                obj = file(fileName, "r+b")
                db = dbf.Dbf(obj)
                return db
            except:
                self.hasCrashedTable = True
        db = dbf.Dbf(fileName, new=True)
        fields_list = (
            ("SHOPINDEX", "C", 6),
            ("CASHNUMBER", "C", 6),
            ("ZNUMBER", "N", 6),
            ("CHECKNUMBE", "N", 6),
            ("PAYMENT", "N", 6),
            ("CARDNUMB", "C", 19),
            ("PAYEDMONEY", "N", 16, 2),
            ("DISCOUNTMO", "N", 16, 2),
            ("PAYEDRUB", "N", 16, 2),
            ("PAYEDCUR", "N", 16, 2),
            ("DISCOUNTRU", "N", 16, 2),
            ("DISCOUNTCU", "N", 16, 2),
            ("DISCCLIRUB", "N", 16, 2),
            ("DISCCLICUR", "N", 16, 2)
        )
        dbf_fields = [self.get_dbf_field(*f) for f in fields_list]
        self.add_fields_to_dbf_reader(db, dbf_fields)
        return db

    def setCashSail(self, SHOPINDEX, CASHNUMBER, ZNUMBER, CHECKNUMBE, ID, DATE, TIME, CARDARTICU, QUANTITY, PRICERUB,
                    TOTALRUB, CASHER, USINGINDEX, REPLACE, OPERATION, CREDCARDIN):
        dbf_item = self.cashSailDB
        rec = self.get_new_record(dbf_item)

        rec['SHOPINDEX'] = SHOPINDEX
        rec['CASHNUMBER'] = CASHNUMBER
        rec['ZNUMBER'] = ZNUMBER
        rec['CHECKNUMBE'] = CHECKNUMBE
        rec['ID'] = ID
        rec['DATE'] = DATE
        rec['TIME'] = int(TIME)
        rec['CARDARTICU'] = CARDARTICU
        rec['CARDSIZE'] = 'NOSIZE'
        rec['QUANTITY'] = QUANTITY
        rec['PRICERUB'] = PRICERUB
        rec['PRICECUR'] = 0
        rec['TOTALRUB'] = TOTALRUB
        rec['TOTALCUR'] = 0
        rec['DEPARTMENT'] = 0
        rec['CASHER'] = CASHER
        rec['USINGINDEX'] = USINGINDEX
        rec['REPLACE'] = REPLACE
        rec['OPERATION'] = OPERATION
        rec['CREDCARDIN'] = CREDCARDIN
        rec['DISCCLIIND'] = 0
        rec['LINKED'] = 0
        self.store_record(dbf_item, rec)

    def setCashDisc(self, SHOPINDEX, CASHNUMBER, ZNUMBER, CHECKNUMBE, ID, DISCOUNTIN, DISCOUNTPR, DISCOUNTRU):
        if self.currentSales:
            return
        dbf_item = self.cashDiscDB
        rec = self.get_new_record(dbf_item)
        rec['SHOPINDEX'] = SHOPINDEX
        rec['CASHNUMBER'] = CASHNUMBER
        rec['ZNUMBER'] = ZNUMBER
        rec['CHECKNUMBE'] = CHECKNUMBE
        rec['ID'] = ID
        rec['DISCOUNTIN'] = DISCOUNTIN
        rec['DISCOUNTPR'] = DISCOUNTPR
        rec['DISCOUNTRU'] = DISCOUNTRU
        rec['DISCOUNTCU'] = 0
        self.store_record(dbf_item, rec)

    def setCurrests(self, counterSales):
        if self.currentSales:
            return
        for shiftNum in counterSales:
            for cashCode in counterSales[shiftNum]:
                value = counterSales[shiftNum][cashCode]

                dbf_item = self.currestsDB
                rec = self.get_new_record(dbf_item)
                rec['SHOPINDEX'] = value['SHOPINDEX']
                rec['CASHNUMBER'] = cashCode
                rec['ZNUMBER'] = shiftNum
                rec['DATE'] = value['DATE']
                rec['SALE'] = value['SALE']
                rec['RETURN'] = value['RETURN']
                rec['STORNO'] = 0
                rec['STORNRET'] = 0
                rec['RESULT'] = value['SALE'] - value['RETURN']
                rec['WASOUTPUT'] = 0
                self.store_record(dbf_item, rec)

    def setCurMoney(self, cashDR):
        if self.currentSales:
            return
        for shiftNum in cashDR:
            for cashCode in cashDR[shiftNum]:
                value = cashDR[shiftNum][cashCode]
                dbf_item = self.curMoneyDB
                rec = self.get_new_record(dbf_item)
                rec['SHOPINDEX'] = value['SHOPINDEX']
                rec['CASHNUMBER'] = cashCode
                rec['ZNUMBER'] = shiftNum
                rec['CURIDX'] = 0
                rec['BEFORE'] = value['BEFORE']  #
                rec['BEFOREIN'] = 0
                rec['BEFOREOUT'] = 0
                rec['PERIODIN'] = value['PERIODIN']
                rec['PERIODOUT'] = value['PERIODOUT']
                rec['AFTER'] = value['AFTER']
                self.store_record(dbf_item, rec)

    def setCashDcrd(self, SHOPINDEX, CASHNUMBER, ZNUMBER, CHECKNUMBE, CARDNUMBER, DISCOUNTRU):
        dbf_item = self.cashDcrdDB
        rec = self.get_new_record(dbf_item)
        rec['SHOPINDEX'] = SHOPINDEX
        rec['CASHNUMBER'] = CASHNUMBER
        rec['ZNUMBER'] = ZNUMBER
        rec['CHECKNUMBE'] = CHECKNUMBE
        rec['CARDTYPE'] = 0
        rec['CARDNUMBER'] = CARDNUMBER
        rec['DISCOUNTRU'] = DISCOUNTRU
        rec['DISCOUNTCU'] = 0
        self.store_record(dbf_item, rec)

    def setCashPay(self, SHOPINDEX, CASHNUMBER, ZNUMBER, CHECKNUMBE, PAYMENT, PAYEDMONEY):
        if self.currentSales:
            return
        dbf_item = self.cashPayDB
        rec = self.get_new_record(dbf_item)
        rec['SHOPINDEX'] = SHOPINDEX
        rec['CASHNUMBER'] = CASHNUMBER
        rec['ZNUMBER'] = ZNUMBER
        rec['CHECKNUMBE'] = CHECKNUMBE
        rec['PAYMENT'] = PAYMENT
        rec['CARDNUMB'] = ''
        rec['PAYEDMONEY'] = PAYEDMONEY
        rec['DISCOUNTMO'] = 0
        rec['PAYEDRUB'] = PAYEDMONEY
        rec['PAYEDCUR'] = 0
        rec['DISCOUNTRU'] = 0
        rec['DISCOUNTCU'] = 0
        rec['DISCCLIRUB'] = 0
        rec['DISCCLICUR'] = 0
        self.store_record(dbf_item, rec)

    def getSaleFiles(self):
        return self.saleFiles

    def get_dbf_field(self, name, field_type, lenght=None, decimal_count=0):
        """Поддержка полей dbf для python2 и python3
        в dbf3 поменялся конструктор для создания полей"""

        # название поля должно быть байтами, не строкой
        name = bytes(str(name).encode("utf-8"))
        # FIXME по-хорошему, тут бы использовать объекты вида DbfIntegerField и т.д.

        # параметр lenght может быть только у character и numeric полей
        if sys.version_info.major == 3:
            if field_type == 'C' or field_type == 'N':
                return field_type, name, lenght, decimal_count
            else:
                return field_type, name
        else:
            if field_type == 'C' or field_type == 'N':
                return name, field_type, lenght, decimal_count
            else:
                return name, field_type

    def add_fields_to_dbf_reader(self, dbf, fields_to_add):
        """Добавление полей в хедер dbf для python2 и python3"""
        if sys.version_info.major == 3:
            dbf.add_field(*fields_to_add)
        else:
            dbf.addField(*fields_to_add)

    def get_new_record(self, dbf):
        """Создание новой записи в таблице dbf для python2 и python3"""
        return dbf.new() if sys.version_info.major == 3 else dbf.newRecord()

    def store_record(self, dbf, rec):
        """Добавление записи в таблицу dbf для python2 и python3"""
        if sys.version_info.major == 3:
            dbf.append(rec)
        else:
            rec.store()
