# -*- coding: UTF-8 -*-

import json
import ujson
import sys
import os
from artix.common import py2to3utils
from datetime import datetime
if os.path.split(sys.path[0])[1] == 'converter':
    sys.path[0] = os.path.normpath('%s/../..' % sys.path[0])
    sys.path.append(os.path.normpath('%s/upload/node' % sys.path[0]))

from common import aif_objects, config, filesmodify, logger, macros
from upload.inserter import InserterError
from artix.common.progress import Progress
maxCountBackupFiles = None
actualCountDaysForBackupFiles = None
errorObjectsCount = 0


def filterKeys(d):
    lowerD = {}
    for k in d.keys():
        if d[k] != None and d[k] != []:
            lowerD[k.lower()] = d[k]
    return lowerD


def countObjects(useQueue, inserter, data, log):
    """
    Подсчитывает количество объектов для загрузки в БД
    Используется при настройке - запись в БД напрямую без очереди
    """
    if useQueue:
        return

    messId = hex(hash(datetime.now()))
    inserter.transactionId = messId
    cntObjects = 0
    for dictFile in data:
        cntObjects += sum(1 for line in open(dictFile) if line.strip() == "---")
    Progress(log).createProgress(messId, cntObjects)


def jsonWrapper(jsonPiece):
    """
    Обертка для ф-ии json.loads
    """
    return json.loads(jsonPiece, object_hook=filterKeys)


def invalidObjectsCount():
    return errorObjectsCount


def general_method(workDirectory, data, node, useQueue=True, extendedProperties={}):
    global errorObjectsCount
    hotKeyByBarcode = node.currentConverter.hotKeyByBarcode
    errorList = []
    log = logger.getUploadLogger("Node", node.id)
    currentObject = None
    try:
        plugin = __import__('upload.inserter.%s' % (node.inserter.name), fromlist='Inserter')
        cl = getattr(plugin, 'Inserter')
        inserter = cl(node)
        listObjects = []
        if data:
            countObjects(useQueue, inserter, data, log)
        result = inserter.start()
        if result and data:
            for dictFile in data:
                dictFile = open(dictFile, 'rb')
                log.info(u'Отправляем объекты справочника в %s...' % (u'очередь' if useQueue else u'БД'))
                jsonPiece = ''
                firstLineCmd = True
                endMarker = False
                beginMarker = False
                jloads = ujson.loads if extendedProperties.get('version', '1') >= '2' else jsonWrapper
                for l in dictFile:
                    line = l.decode() if sys.version_info.major == 3 else l
                    if not len(listObjects) and line.strip() == "### data begin ###":
                        log.info(u"Найден маркер начала справочника")
                        beginMarker = True
                    elif beginMarker and line.strip() == "### data end ###":
                        log.info(u"Найден маркер конца справочника")
                        endMarker = True
                    elif line.strip() == '---':
                        if jsonPiece.strip() != '':
                            currentObject = jsonPiece
                            dictObject = jloads(jsonPiece)
                            if hotKeyByBarcode:
                                if 'addHotKey' == dictObject['command'] and 'bybarcode' not in dictObject['hotkey']:
                                    dictObject['hotkey']['bybarcode'] = hotKeyByBarcode
                            listObjects.append(dictObject)
                        jsonPiece = ''
                    else:
                        if firstLineCmd:
                            startPos = line.find('{')
                            if startPos != -1:
                                firstLineCmd = False
                                line = line[startPos:]
                            else:
                                line = ''
                        jsonPiece += ' ' + str(line)
                    if len(listObjects) == node.pack_size:
                        inserter.add_package(listObjects)
                        listObjects = []

                if beginMarker and not endMarker:
                    currentObject = None
                    raise Exception(u'Не найден маркер конца справочника')

                if len(listObjects) > 0:
                    inserter.add_package(listObjects)
                    listObjects = []
                dictFile.close()
            result = inserter.stop()
            errorObjectsCount = inserter.invalidObjectsCount()
        if result and not data:
            result = inserter.stop()
        return result, errorList
    except InserterError:
        raise
    except aif_objects.AifObjectNotValid as e:
        log.error(u'Ошибка: %s' % e)
        errorList.append(u'Ошибка: %s' % e)
    except Exception as e:
        log.error(u'Ошибка: %s' % e)
        if currentObject:
            log.error('Ошибка произошла на объекте: %s' % py2to3utils.ensureStr(currentObject, 'utf-8'))
            errorList.append('Ошибка произошла на объекте: %s' % py2to3utils.ensureStr(currentObject, 'utf-8'))
        else:
            errorList.append(u'Ошибка: %s' % e)
    return False, errorList


def convert(workDirectory, node, useQueue=True, extendedProperties={}):
    workDirectory = os.path.normpath('%s/%s/aif' % (workDirectory, node.id))
    if not os.path.exists(workDirectory):
        os.makedirs(workDirectory)
    if node.currentConverter.flags:
        filesmodify.moveFiles(node.currentConverter.flags, workDirectory)
    data = filesmodify.moveFiles(node.currentConverter.data, workDirectory) if node.currentConverter.data else None
    result, errorList = general_method(workDirectory, data, node, useQueue, extendedProperties)

    log = logger.getUploadLogger("Node", node.id)
    if result and data:
        filesmodify.archive(log, workDirectory, node.backup_dir, maxCountFiles=maxCountBackupFiles, actualCountDays=actualCountDaysForBackupFiles)

    return result, errorList


def insertInDB(propertyReader, dictFile, extendedProperties):
    """
    Подготавливает ноду, inserter для передачи в NES и загрузку в БД
    без использования очереди.
    Функция вызывается из sync агента напрямую
    """
    from node import Node, AifConverterParameters, DBInserterConfig
    global maxCountBackupFiles
    global actualCountDaysForBackupFiles
    if os.path.split(sys.path[0])[1] == 'converter':
        sys.path[0] = os.path.normpath('%s/../..' % sys.path[0])
        sys.path.append(os.path.normpath('%s/upload/node' % sys.path[0]))

    shopCode = propertyReader.storeID
    log = logger.getUploadLogger("Node", shopCode)
    log.info(u'Загрузка данных напрямую в БД')

    backupDir = macros.applyMacros(config.getString('UPLOAD', 'backupDir', '/linuxcash/cash/data/backup/dicts'), log, shopcode=shopCode)
    maxCountBackupFiles = config.getInt('UPLOAD', 'maxCountBackupFiles', 30)
    actualCountDaysForBackupFiles = config.getInt('UPLOAD', 'actualCountDaysForBackupFiles', 30)
    workDirectory = config.getString('main', 'workDirectory', '/linuxcash/cash/data/tmp/exchangers')

    pack_size = config.getInt('FILEQUEUE', 'queueChunkSize', 25000)
    if not os.path.exists(workDirectory):
        os.makedirs(workDirectory)

    flagName = dictFile + '.flag' if dictFile else None
    open(flagName, 'w+').close()
    node = Node(id=shopCode,
            backup_dir=backupDir,
            converters=[AifConverterParameters(flags=[flagName] if flagName else None, data=[dictFile] if dictFile else None)],
            pack_size=pack_size,
            inserter=DBInserterConfig(),
            badDictsDirectory=macros.applyMacros(config.getString('UPLOAD', 'badDictsDirectory', '/linuxcash/cash/data/backup/badDicts').strip(), log, shopcode=shopCode),
            extendedProperties=extendedProperties)

    node.currentConverter = node.converters[0]
    node.currentConverter.dict_state_id = extendedProperties.get('dictCashLoadStateId', None)
    node.cashCode = propertyReader.cashCode
    node.currentConverter.statusServer = '%s:%s' % (propertyReader.remoteIp, propertyReader.cashServerRestPort)
    log.info(u'Начинаем конвертирование')
    return convert(workDirectory, node, False, extendedProperties)
