# -*- coding: UTF-8 -*-

from common import aif_objects as aif, logger
from upload.inserter import InserterError

from xml.etree.ElementTree import iterparse

errorObjectsCount = 0

class Converter(object):

    def __init__(self, workDirectory, data, node):
        self.workDirectory = workDirectory
        self.data = data
        self.node = node
        self.log = logger.getUploadLogger("Node", node.id)

        self.plugin = __import__('upload.inserter.%s' % (node.inserter.name), fromlist='Inserter')
        cl = getattr(plugin, 'Inserter')
        self.inserter = cl(node)
        self.listObjects = []
        self.measureCodes = {'EA':1, 'G':2, 'KG':3, 'L':4, 'M':5, 'M2':6}
        self.ns = {'ns0':'http://www.oracle.com/retail/integration/rib/RibMessages',
                'ns1':'http://www.oracle.com/retail/integration/payload/ItemDesc',
                'ns2':'http://www.oracle.com/retail/integration/payload/ItemHdrDesc',
                'ns3':'http://www.oracle.com/retail/integration/payload/ItemUPCDesc',
                'ns4':'http://www.oracle.com/retail/integration/payload/ItemRef',
                'ns5':'http://www.oracle.com/retail/integration/payload/ItemUPCRef',
                'ns6':'http://www.oracle.com/retail/integration/payload/RegPrcChgDesc',
                'ns7':'http://www.oracle.com/retail/integration/payload/RegPrcChgDtl'
                }

    def appendDict(self, aif_object):
        self.listObjects.append(aif_object)
        if len(self.listObjects) == self.node.pack_size:
            self.inserter.add_package(self.listObjects)
            self.listObjects = []

    # Добавление товаров
    def convertAddInvent(self, messageData):
        itemHdrDesc = messageData.find('{%(ns1)s}ItemDesc/{%(ns2)s}ItemHdrDesc' % self.ns)
        status = itemHdrDesc.find('{%(ns2)s}status' % self.ns).text
        merchandise_ind = itemHdrDesc.find('{%(ns2)s}merchandise_ind' % self.ns).text

        if status == 'A' and merchandise_ind == 'Y':
            inventcode = itemHdrDesc.find('{%(ns2)s}item' % self.ns).text
            name = itemHdrDesc.find('{%(ns2)s}item_desc' % self.ns).text
            #rtext = itemHdrDesc.find('{%(ns2)s}short_desc' % self.ns).text
            measurecode = self.measureCodes.get(itemHdrDesc.find('{%(ns2)s}standard_uom' % self.ns).text)
            price = itemHdrDesc.find('{%(ns2)s}unit_retail' % self.ns).text
            return aif.Invent().addCmd(inventcode = inventcode, barcode = None, name = name, measurecode = measurecode, price = price)

    # Добавление дочерних товаров(дополнительных штрих-кодов)
    def convertAddBarcode(self, messageData):
        itemUPCDesc = messageData.find('{%(ns3)s}ItemUPCDesc' % self.ns)
        #print 'itemUPCDesc = %s' % itemUPCDesc
        barcode = itemUPCDesc.find('{%(ns3)s}item_id' % self.ns).text
        inventcode = itemUPCDesc.find('{%(ns3)s}item_parent' % self.ns).text
        return aif.Barcode().addCmd(code = inventcode, barcode = barcode)

    # Удаление товаров
    def convertDelInvent(self, messageData):
        itemRef = messageData.find('{%(ns4)s}ItemRef' % self.ns)
        inventcode = itemRef.find('{%(ns4)s}item' % self.ns).text
        return aif.Invent().deleteCmd(inventcode = inventcode)

    # Удаление дочерних товаров(дополнительных штрих-кодов)
    def convertDelBarcode(self, messageData):
        itemRef = messageData.find('{%(ns5)s}ItemUPCRef/{%(ns4)s}ItemRef' % self.ns)
        barcode = itemRef.find('{%(ns4)s}item' % self.ns).text
        return aif.Barcode().deleteCmd(barcode = barcode)

    # Добавление переоценок и распродаж (doctype=1 - переоценка | doctype=2 - распродажа)
    def convertAddPrice(self, messageData, doctype = 1):
        documentidName = 'price_change_id' if doctype == 1 else 'clearance_id'
        regPrcChgDesc = messageData.find('{%(ns6)s}RegPrcChgDesc' % self.ns)
#        for regPrcChgDtl RegPrcChgDtl
        #print 'itemUPCDesc = %s' % itemUPCDesc
        barcode = itemUPCDesc.find('{%(ns7)s}item' % self.ns).text
        documentid = itemUPCDesc.find('{%(ns7)%s' % (self.ns, documentidName)).text
        effectivedate = itemUPCDesc.find('{%(ns7)effective_date' % self.ns).text
        price = itemUPCDesc.find('{%(ns7)selling_unit_retail' % self.ns).text
        return aif.Price().addCmd(barcode = barcode, documentid = documentid, effectivedate = effectivedate, doctype = doctype, price = price)

    # Удаление переоценок и распродаж (doctype=1 - переоценка | doctype=2 - распродажа)
    def convertDelPrice(self, messageData, doctype = 1):
        documentidName = 'price_change_id' if doctype == 1 else 'clearance_id'
        itemRef = messageData.find('{%(ns7)s}ItemUPCRef/{%(ns4)s}ItemRef' % self.ns)
        barcode = itemUPCDesc.find('{%(ns7)s}item' % self.ns).text
        documentid = itemUPCDesc.find('{%(ns7)%s' % (self.ns, documentidName)).text
        return aif.Price().deleteCmd(barcode = barcode, documentid = documentid, doctype = doctype)

    def run(self):
        global errorObjectsCount
        result = self.inserter.start()
        if result:
            for dictFile in self.data:
                iparse = iterparse(dictFile, ['start', 'end'])
                ribMessages = None
                for event, elem in iparse:
                    if event == 'start' and elem.tag == '{%(ns0)s}RibMessages' % self.ns:
                        ribMessages = elem
                        break

                messages = (elem for event, elem in iparse
                                if event == 'end' and elem.tag == '{%(ns0)s}ribMessage' % self.ns)
                for ribMessage in messages:
                    messageType = ribMessage.find('{%(ns0)s}type' % self.ns).text
                    messageData = ribMessage.find('{%(ns0)s}messageData' % self.ns)
                    self.log.info('messageType = %s' % messageType)
                    currentObj = None
                    if messageType == 'ItemCre':
                        currentObj = self.convertAddInvent(messageData)
                    elif messageType == 'ItemUPCCre' or messageType == 'ItemUPCMod':
                        currentObj = self.convertAddBarcode(messageData)
                    elif messageType == 'ItemDel':
                        currentObj = self.convertDelInvent(messageData)
                    elif messageType == 'ItemUPCDel':
                        currentObj = self.convertDelBarcode(messageData)
                    elif messageType == 'Regprcchgcre' or messageType == 'Regprcchgmod':
                        currentObj = self.convertAddPrice(messageData, doctype=1)
                    elif messageType == 'Regprcchgdel':
                        currentObj = self.convertDelPrice(messageData, doctype=1)
                    elif messageType == 'clrprcchgcre' or messageType == 'clrprcchgmod':
                        currentObj = self.convertAddPrice(messageData, doctype=2)
                    elif messageType == 'clrprcchgdel':
                        currentObj = self.convertDelPrice(messageData, doctype=2)

                    self.appendDict(currentObj)
                    ribMessages.remove(ribMessage)
                if len(self.listObjects) > 0:
                    self.inserter.add_package(self.listObjects)
                    self.listObjects = []
            result = self.inserter.stop()
            errorObjectsCount = self.inserter.invalidObjectsCount()
        return result


def general_method(workDirectory, data, node):
    log = logger.getUploadLogger("Node", node.id)
    result = False
    errorList = []
    try:
        result = Converter(workDirectory, data, node).run()
    except InserterError as e:
        raise
    except Exception as e:
        errorList.append(e)
        log.exception(e)
    return result, errorList

def invalidObjectsCount():
    return errorObjectsCount
