# -*- coding: UTF-8 -*-
import itertools
from itertools import groupby

from common import aif_objects as dicts
from upload.converter.smagPlugins.xml.artix_mapper.ukm_tmc_to_artix_tmc_mapper import UkmToArtixTmcMapper

"""
Вспомогательный класс для получения команд на загрузку aif-объектов
"""


class ConverterUkm(object):

    def __init__(self):
        self.commands = {
            "updateItems": lambda xml_tag: self.tmc_handler(**xml_tag),
            "storePrices": lambda xml_tag: self.price_handler(**xml_tag),
            "storePricesAdd": lambda xml_tag: self.tprice_handler(**xml_tag),
            "taxes": lambda xml_tag: self.tax_handler(**xml_tag),
            "alcocode": lambda xml_tag: self.alcocode_handler(**xml_tag),
            "classif": lambda xml_tag: self.group_handler(**xml_tag),
            "kit": lambda xml_tag: self.kit_handler(**xml_tag),
            "storeCashiers": lambda xml_tag: self.cashiers_handler(**xml_tag),
            "picklists": lambda xml_tag: self.picklist_handler(**xml_tag)
        }
        self.measureMapper = {}
        self.quantityOptionsMapper = {}
        self.tmcRestrictionsMapper = {}

    def get_command(self, line, data, kwargs=None):
        if kwargs is None:
            kwargs = {}
        kwargs['data'] = data
        return self.commands[line](kwargs)

    def tmc_handler(self, data, hasClearFlag):
        commands = []
        if hasClearFlag:
            commands.append(dicts.Invent().clearCmd())
        tmcs = UkmToArtixTmcMapper.map_tmc(data, self.measureMapper, self.quantityOptionsMapper, self.tmcRestrictionsMapper)
        commands.extend([dicts.Invent().addCmd(**tmc) for tmc in tmcs])
        return commands

    def alcocode_handler(self, data, hasClearFlag):
        commands = []
        if hasClearFlag:
            commands.append(dicts.AlcoCode().clearCmd())
            commands.append(dicts.TmcAlcocode().clearCmd())
        alcocodes_gen = UkmToArtixTmcMapper.map_alcocode(data)
        alcocodes, tmc_alcocodes = itertools.tee(alcocodes_gen)
        # на основе полученных алкокодов формируем команды на добавление в таблицы alcocode и tmcalcocode
        add_alcocodes = [dicts.AlcoCode().addCmd(code["alcocode"]) for code in alcocodes]  # type:list
        add_tmc_alcocodes = [dicts.TmcAlcocode().addCmd(**code) for code in tmc_alcocodes]  # type:list
        add_alcocodes.extend(add_tmc_alcocodes)
        commands.extend(add_alcocodes)
        return commands

    def group_handler(self, data, hasClearFlag):
        commands = []
        if hasClearFlag:
            commands.append(dicts.InventGroup().clearCmd())
        groups = UkmToArtixTmcMapper.map_groups(data)
        commands.extend([dicts.InventGroup().addCmd(**group) for group in groups])
        return commands

    def kit_handler(self, data, hasClearFlag):
        commands = []
        if hasClearFlag:
            commands.append(dicts.TmcKit().clearCmd())
            commands.append(dicts.TmcKitItem().clearCmd())
        kits, kit_items = UkmToArtixTmcMapper.map_kit(data)
        # формируем команды на добавление в таблицы tmckit и tmckititem
        kit_add_cmds = [dicts.TmcKit().addCmd(**kit) for kit in kits]  # type: list
        kit_items_add_cmd = [dicts.TmcKitItem().addCmd(**kit_item) for kit_item in kit_items]  # type: list
        kit_add_cmds.extend(kit_items_add_cmd)
        commands.extend(kit_add_cmds)
        return commands

    def price_handler(self, data, hasClearFlag):
        commands = []
        if hasClearFlag:
            commands.append(dicts.Price().clearCmd())
        prices = UkmToArtixTmcMapper.map_prices(data)
        commands.extend([dicts.Price().addCmd(**p) for p in prices])
        return commands

    def tprice_handler(self, data, hasClearFlag):
        commands = []
        if hasClearFlag:
            commands.append(dicts.AdditionalPrice().clearCmd())
        tprices = UkmToArtixTmcMapper.map_tprices(data)
        commands.extend([dicts.AdditionalPrice().addCmd(**p) for p in tprices])
        return commands

    def tax_handler(self, data, hasClearFlag):
        commands = []
        if hasClearFlag:
            commands.append(dicts.TaxGroup().clearCmd())
        taxes = UkmToArtixTmcMapper.map_tax(data)
        # получаем список вида taxgroupid -> [taxes]
        mapped_taxes = groupby(taxes, lambda x: x["code"])
        commands.extend([dicts.TaxGroup().addCmd(idtaxgroup=taxgroupid,
                                                 taxes=[{'name': tax["name"], 'rate': int(tax["rate"])} for tax in
                                                        taxes])
                         for taxgroupid, taxes in mapped_taxes])
        return commands

    def cashiers_handler(self, data, hasClearFlag):
        commands = []
        if hasClearFlag:
            commands.append(dicts.MCashUser().clearCmd())
        cashiers = UkmToArtixTmcMapper.map_cashier(data)
        commands.extend([dicts.MCashUser().addCmd(**user) for user in cashiers])
        return commands # type: list

    def picklist_handler(self, data, hasClearFlag):
        commands = []
        if hasClearFlag:
            commands.append(dicts.Picklist().clearCmd())
        picklists = UkmToArtixTmcMapper.map_picklist(data)
        commands.extend([dicts.Picklist().addCmd(**picklist) for picklist in picklists])
        # формируем команды на добавление в таблицу picklist
        return commands  # type: list
